use crate::config::config::Config;
use std::fs::{create_dir_all, DirBuilder, File};
use std::path::PathBuf;

pub struct InitArgs {}

/// Creates the necessary files and directories for a nextsync repository.
///
/// The following files and directories are created:
/// - `.nextsync/objects`
/// - `.nextsync/refs`
/// - `.nextsync/index`
/// - `.nextsync/HEAD`
///
/// If the `execution_path` specified in the `config` does not exist, it will be created.
///
/// # Panics
///
/// This function will panic if it cannot create the mentioned files or directories.
pub fn exec(_: InitArgs, config: Config) {
    let mut path: PathBuf = config.execution_path.clone();
    path.push(".nextsync");

    if config.execution_path.exists() {
        // Check the dir is not already a nextsync dir
        if path.exists() {
            panic!(
                "{} is already a nextsync directory, cannot overwrite it!",
                config.execution_path.display()
            );
        }
    }

    let mut binding = DirBuilder::new();
    let mut builder = binding.recursive(true);

    for dir in &["objects", "refs"] {
        path.push(dir);
        builder.create(&path).unwrap();
        path.pop();
    }

    for file in &["HEAD", "index"] {
        path.push(file);
        File::create(&path).unwrap();
        path.pop();
    }
}
